﻿using System;
using System.Collections.Generic;
using System.Linq;
using Verse;
using RimWorld;
using Verse.AI;
using System.IO;

namespace Nandonalt_ColonyLeadership
{
    public class IncidentWorker_Rebellion : IncidentWorker
    {
        private const int FixedPoints = 30;

        public static void removeLeadership(Pawn current)
        {
            string sciLeaderDeffName = Nandonalt_ColonyLeadership.Config.ConfigManager.getScientistLeaderDeffName();
            HediffLeader h1 = (HediffLeader)current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("leader1"));
            HediffLeader h2 = (HediffLeader)current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("leader2"));
            HediffLeader h3 = (HediffLeader)current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("leader3"));
            HediffLeader h4 = (HediffLeader)current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named(sciLeaderDeffName));

            if (h2 != null) h1 = h2;
            if (h3 != null) h1 = h3;
            if (h4 != null) h1 = h4;


            Hediff hediff = HediffMaker.MakeHediff(HediffDef.Named("leaderExpired"), current, null);
            hediff.Severity = 1f;
            HediffLeader hl = (HediffLeader)hediff;
            hl.ticksLeader = h1.ticksLeader;
            current.health.AddHediff(hl, null, null);
            current.health.RemoveHediff(h1);
            current.needs.AddOrRemoveNeedsAsAppropriate();

        }

        protected override bool TryExecuteWorker(IncidentParms parms)
        {

            //try
            //{


                Map map = (Map)parms.target;

                List<Pawn> pawns = IncidentWorker_SetLeadership.getAllColonists();
                List<Pawn> tpawns2 = new List<Pawn>();

            string sciLeaderDeffName = Nandonalt_ColonyLeadership.Config.ConfigManager.getScientistLeaderDeffName();
            foreach (Pawn current in pawns)
                {
                    Hediff h1 = current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("leader1"));
                    Hediff h2 = current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("leader2"));
                    Hediff h3 = current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("leader3"));
                    Hediff h4 = current.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named(sciLeaderDeffName));
                    if (h1 == null && h2 == null && h3 == null && h4 == null && current.Map == map) { tpawns2.Add(current); }
                }
                Pawn pawn = tpawns2.RandomElement();

                if (pawn != null && !pawn.Downed && !pawn.Dead)
                {
                    if (pawn.CurJob != null && pawn.jobs.curDriver.asleep != true)
                    {
                        pawn.jobs.EndCurrentJob(JobCondition.InterruptForced, true);
                    }
                    pawn.mindState.mentalStateHandler.TryStartMentalState(DefDatabase<MentalStateDef>.GetNamed("Rebelling"), null, false, false, null);
                    String s = "He";
                    if (pawn.gender == Gender.Female) s = "She";

                    //                Find.LetterStack.ReceiveLetter("RebelLetter".Translate(), "RebelLetterDesc".Translate(new object[] { pawn.Name.ToStringShort }), LetterDefOf.ThreatBig, pawn, null);

                    Find.LetterStack.ReceiveLetter("RebelLetter".Translate(), "RebelLetterDesc".Translate(new object[] { pawn.Name.ToStringShort }), LetterDefOf.NegativeEvent, pawn, null);

                    return true;

                }

                return false;
            /**}
            catch (Exception ex)
            {
                System.IO.File.WriteAllText("C:/Logs/debugLog.txt", ex.Message.ToString());
                
                return false;
            }**/
        }
    }
}