﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.Sound;
using System.Collections;
using System.IO;

namespace Nandonalt_ColonyLeadership
{
    public class Dialog_ChooseRules : Window
    {
        protected string curName;
        public GovType chosenLeadership;
        public String electionFrequency;
        
        private bool updateLabel = false;
        public List<String> electFrequencyOptions = new List<String>(new string[] {"3", "2", "1"});
        public int MaxSize;
        public int MinSize;
        public string MaxSizebuf;
        public string MinSizebuf;
        public bool permanent;
        
        public override Vector2 InitialSize
        {
            get
            {
                return new Vector2(360f, 360f);
            }
        }

        public Dialog_ChooseRules()
        {

            this.forcePause = true;
            this.doCloseX = false;
            this.absorbInputAroundWindow = true;
            this.closeOnClickedOutside = false;
            this.chosenLeadership = ColonyLeadership.govtypes[0];
            this.electionFrequency = "3";
            
        }


        public override void DoWindowContents(Rect inRect)
        {
            Text.Font = GameFont.Small;

            Listing_Standard listing_Standard = new Listing_Standard();
            listing_Standard.Begin(inRect);

            listing_Standard.Label("ChooseGov".Translate());

            String label = this.chosenLeadership.name;
            if (listing_Standard.ButtonText(label, null))
            {
                List<FloatMenuOption> list = new List<FloatMenuOption>();

                foreach (GovType gov in ColonyLeadership.govtypes)
                {
                    list.Add(new FloatMenuOption(gov.name, delegate
                    {
                        this.chosenLeadership = gov;
                    }, MenuOptionPriority.Default, delegate () { TooltipHandler.TipRegion(inRect, gov.desc); }, null, 0f, null, null));
                }
                Find.WindowStack.Add(new FloatMenu(list, "Government Types", false));
            }

            listing_Standard.Gap(24f);
            listing_Standard.Label("Elections per Season");

            if (!updateLabel)
            {
                this.electionFrequency = Config.ConfigManager.getElectionDays().Count.ToString();
            }
            String freqLabel = this.electionFrequency;

            
            
            if (listing_Standard.ButtonText(freqLabel, null))
            {
                List<FloatMenuOption> electFreqList = new List<FloatMenuOption>();
                foreach (String f in this.electFrequencyOptions)
                {
                    electFreqList.Add(new FloatMenuOption(f, delegate
                    {
                        this.electionFrequency = f;
                        freqLabel = f;
                        updateLabel = true;
                    }, MenuOptionPriority.Default, delegate () { TooltipHandler.TipRegion(inRect, "PerSeason"); }, null, 0f, null, null));
                }
                Find.WindowStack.Add(new FloatMenu(electFreqList, "election frequency", false));
            }

            
            listing_Standard.Gap(24f);
            listing_Standard.Label("Scientist Psychic Sensitivity");
            bool isScientistSensitiveToPsychic = Nandonalt_ColonyLeadership.Config.ConfigManager.isPsychicSensitivityForScientist();
            String sensitivityLabel = (isScientistSensitiveToPsychic) ? "ON" : "OFF";
            
            if (listing_Standard.ButtonText(sensitivityLabel, null))
            {
                List<FloatMenuOption> sensitivityOptions = new List<FloatMenuOption>();

                sensitivityOptions.Add(new FloatMenuOption("ON", delegate
                {
                    Nandonalt_ColonyLeadership.Config.ConfigManager.setPsychicSensitivityForScientist(true);
                    updateLabel = true;
                }, MenuOptionPriority.Default, delegate () { TooltipHandler.TipRegion(inRect, "Adds Buffs to medical surgery and negation if turned off."); }, null, 0f, null, null));

                sensitivityOptions.Add(new FloatMenuOption("OFF", delegate
                {
                    Nandonalt_ColonyLeadership.Config.ConfigManager.setPsychicSensitivityForScientist(false);
                    updateLabel = true;
                }, MenuOptionPriority.Default, delegate () { TooltipHandler.TipRegion(inRect, "Adds Buffs to medical surgery and negation if turned off."); }, null, 0f, null, null));

                Find.WindowStack.Add(new FloatMenu(sensitivityOptions, "Science Leader Psycic Sensitivty", false));
            }


            listing_Standard.Gap(24f);
            if (listing_Standard.ButtonText("OK".Translate(), null))
            {
                int freq = int.Parse(this.electionFrequency);
                GameComponent_ColonyLeadership comp = Utility.getCLComp();
                if (comp != null) comp.chosenLeadership = this.chosenLeadership;
                if (comp != null) comp.chosenGov = ColonyLeadership.govtypes.IndexOf(this.chosenLeadership);
                Find.WindowStack.TryRemove(this, true);
                if (this.chosenLeadership.name == "Dictatorship".Translate() || this.chosenLeadership.name == "Monarchy")
                {
                    foreach (Pawn p in IncidentWorker_SetLeadership.getAllColonists())
                    {
                        LeaderWindow.purgeLeadership(p);
                    }

                    Find.WindowStack.Add(new Dialog_ChooseLeader());
                }
                else
                {
                    List<int> newDays = getElectionDays(freq);
                    Nandonalt_ColonyLeadership.Config.ConfigManager.setElectionsDays(getElectionDays(freq));
                }
                updateLabel = false;
            }
            listing_Standard.End();
        }

        private List<int> getElectionDays(int freq)
        {
            List<int> allowedDays = new List<int>();
             allowedDays.Add(0);

            if (freq == 2)
            {
                allowedDays.Add(10);
            }
            else if(freq == 3)
            {
                allowedDays.Add(6);
                allowedDays.Add(14);
            }

            return allowedDays;
        }
    }

}
